@@ -936,6 +936,16 @@ class TestMinitestAssertions < Minitest::Test
     end
   end
 
+  def test_assert_respond_to__include_all
+    @tc.assert_respond_to @tc, :exit, include_all: true
+  end
+
+  def test_assert_respond_to__include_all_triggered
+    assert_triggered(/Expected .+::DummyTest. to respond to #exit\?/) do
+      @tc.assert_respond_to @tc, :exit?, include_all: true
+    end
+  end
+
   def test_assert_same
     @assertion_count = 3
 
@@ -1062,6 +1072,66 @@ class TestMinitestAssertions < Minitest::Test
     end
   end
 
+  def test_assert_pattern
+    if RUBY_VERSION > "3" then
+      @tc.assert_pattern do
+        exp = if RUBY_VERSION.start_with? "3.0"
+                "(eval):1: warning: One-line pattern matching is experimental, and the behavior may change in future versions of Ruby!\n"
+              else
+                ""
+              end
+        assert_output nil, exp do
+          eval "[1,2,3] => [Integer, Integer, Integer]" # eval to escape parser for ruby<3
+        end
+      end
+    else
+      @assertion_count = 0
+
+      assert_raises NotImplementedError do
+        @tc.assert_pattern do
+          # do nothing
+        end
+      end
+    end
+  end
+
+  def test_assert_pattern_traps_nomatchingpatternerror
+    skip unless RUBY_VERSION > "3"
+    exp = if RUBY_VERSION.start_with? "3.0" then
+            "[1, 2, 3]" # terrible error message!
+          else
+            /length mismatch/
+          end
+
+    assert_triggered exp do
+      @tc.assert_pattern do
+        capture_io do # 3.0 is noisy
+          eval "[1,2,3] => [Integer, Integer]" # eval to escape parser for ruby<3
+        end
+      end
+    end
+  end
+
+  def test_assert_pattern_raises_other_exceptions
+    skip unless RUBY_VERSION >= "3.0"
+
+    @assertion_count = 0
+
+    assert_raises RuntimeError do
+      @tc.assert_pattern do
+        raise "boom"
+      end
+    end
+  end
+
+  def test_assert_pattern_with_no_block
+    skip unless RUBY_VERSION >= "3.0"
+
+    assert_triggered "assert_pattern requires a block to capture errors." do
+      @tc.assert_pattern
+    end
+  end
+
   def test_capture_io
     @assertion_count = 0
 
@@ -1093,18 +1163,14 @@ class TestMinitestAssertions < Minitest::Test
   def test_class_asserts_match_refutes
     @assertion_count = 0
 
-    methods = Minitest::Assertions.public_instance_methods
-    methods.map!(&:to_s) if Symbol === methods.first
+    methods = Minitest::Assertions.public_instance_methods.map(&:to_s)
 
     # These don't have corresponding refutes _on purpose_. They're
     # useless and will never be added, so don't bother.
     ignores = %w[assert_output assert_raises assert_send
                  assert_silent assert_throws assert_mock]
 
-    # These are test/unit methods. I'm not actually sure why they're still here
-    ignores += %w[assert_no_match assert_not_equal assert_not_nil
-                  assert_not_same assert_nothing_raised
-                  assert_nothing_thrown assert_raise]
+    ignores += %w[assert_allocations] # for minitest-gcstats
 
     asserts = methods.grep(/^assert/).sort - ignores
     refutes = methods.grep(/^refute/).sort - ignores
@@ -1314,6 +1380,56 @@ class TestMinitestAssertions < Minitest::Test
     end
   end
 
+  def test_refute_pattern
+    if RUBY_VERSION >= "3.0"
+      @tc.refute_pattern do
+        capture_io do # 3.0 is noisy
+          eval "[1,2,3] => [Integer, Integer, String]"
+        end
+      end
+    else
+      @assertion_count = 0
+
+      assert_raises NotImplementedError do
+        @tc.refute_pattern do
+          eval "[1,2,3] => [Integer, Integer, String]"
+        end
+      end
+    end
+  end
+
+  def test_refute_pattern_expects_nomatchingpatternerror
+    skip unless RUBY_VERSION > "3"
+
+    assert_triggered(/NoMatchingPatternError expected, but nothing was raised./) do
+      @tc.refute_pattern do
+        capture_io do # 3.0 is noisy
+          eval "[1,2,3] => [Integer, Integer, Integer]"
+        end
+      end
+    end
+  end
+
+  def test_refute_pattern_raises_other_exceptions
+    skip unless RUBY_VERSION >= "3.0"
+
+    @assertion_count = 0
+
+    assert_raises RuntimeError do
+      @tc.refute_pattern do
+        raise "boom"
+      end
+    end
+  end
+
+  def test_refute_pattern_with_no_block
+    skip unless RUBY_VERSION >= "3.0"
+
+    assert_triggered "refute_pattern requires a block to capture errors." do
+      @tc.refute_pattern
+    end
+  end
+
   def test_refute_predicate
     @tc.refute_predicate "42", :empty?
   end
@@ -1334,6 +1450,16 @@ class TestMinitestAssertions < Minitest::Test
     end
   end
 
+  def test_refute_respond_to__include_all
+    @tc.refute_respond_to "blah", :missing, include_all: true
+  end
+
+  def test_refute_respond_to__include_all_triggered
+    assert_triggered(/Expected .*DummyTest.* to not respond to exit./) do
+      @tc.refute_respond_to @tc, :exit, include_all: true
+    end
+  end
+
   def test_refute_same
     @tc.refute_same 1, 2
   end
