@@ -1,35 +1,36 @@
 require "minitest/autorun"
 require "minitest/metametameta"
+require "forwardable"
 
-class TestMinitestReporter < Minitest::Test
+class Runnable
+  def woot
+    assert true
+  end
+end
+
+class TestMinitestReporter < MetaMetaMetaTestCase
 
   attr_accessor :r, :io
 
   def new_composite_reporter
+    # Ruby bug in older versions of 2.2 & 2.3 on all platforms
+    # Latest Windows builds were 2.2.6 and 2.3.3.  Latest Ruby releases were
+    # 2.2.10 and 2.3.8.
+    skip if windows? && RUBY_VERSION < '2.4'
     reporter = Minitest::CompositeReporter.new
     reporter << Minitest::SummaryReporter.new(self.io)
     reporter << Minitest::ProgressReporter.new(self.io)
 
-    def reporter.first
-      reporters.first
-    end
-
-    def reporter.results
-      first.results
-    end
-
-    def reporter.count
-      first.count
-    end
-
-    def reporter.assertions
-      first.assertions
-    end
+    # eg reporter.results -> reporters.first.results
+    reporter.extend Forwardable
+    reporter.delegate :first => :reporters
+    reporter.delegate %i[results count assertions options to_s] => :first
 
     reporter
   end
 
   def setup
+    super
     self.io = StringIO.new("")
     self.r  = new_composite_reporter
   end
@@ -42,10 +43,30 @@ class TestMinitestReporter < Minitest::Test
                                                     rescue => e
                                                       e
                                                     end)
+      @et = Minitest::Result.from @et
     end
     @et
   end
 
+  def system_stack_error_test
+    unless defined? @sse then
+
+      ex = SystemStackError.new
+
+      pre  = ("a".."c").to_a
+      mid  = ("aa".."ad").to_a * 67
+      post = ("d".."f").to_a
+      ary  = pre + mid + post
+
+      ex.set_backtrace ary
+
+      @sse = Minitest::Test.new(:woot)
+      @sse.failures << Minitest::UnexpectedError.new(ex)
+      @sse = Minitest::Result.from @sse
+    end
+    @sse
+  end
+
   def fail_test
     unless defined? @ft then
       @ft = Minitest::Test.new(:woot)
@@ -54,12 +75,19 @@ class TestMinitestReporter < Minitest::Test
                         rescue Minitest::Assertion => e
                           e
                         end
+      @ft = Minitest::Result.from @ft
     end
     @ft
   end
 
   def passing_test
-    @pt ||= Minitest::Test.new(:woot)
+    @pt ||= Minitest::Result.from Minitest::Test.new(:woot)
+  end
+
+  def passing_test_with_metadata
+    test = Minitest::Test.new(:woot)
+    test.metadata[:meta] = :data
+    @pt ||= Minitest::Result.from test
   end
 
   def skip_test
@@ -70,18 +98,43 @@ class TestMinitestReporter < Minitest::Test
                       rescue Minitest::Assertion => e
                         e
                       end
+      @st = Minitest::Result.from @st
     end
     @st
   end
 
+  def test_to_s
+    r.record passing_test
+    r.record fail_test
+    assert_match "woot", r.to_s
+  end
+
+  def test_options_skip_F
+    r.options[:skip] = "F"
+
+    r.record passing_test
+    r.record fail_test
+
+    refute_match "woot", r.to_s
+  end
+
+  def test_options_skip_E
+    r.options[:skip] = "E"
+
+    r.record passing_test
+    r.record error_test
+
+    refute_match "RuntimeError: no", r.to_s
+  end
+
   def test_passed_eh_empty
-    assert r.passed?
+    assert_predicate r, :passed?
   end
 
   def test_passed_eh_failure
     r.results << fail_test
 
-    refute r.passed?
+    refute_predicate r, :passed?
   end
 
   SKIP_MSG = "\n\nYou have skipped tests. Run with --verbose for details."
@@ -91,7 +144,7 @@ class TestMinitestReporter < Minitest::Test
 
     r.results << error_test
 
-    refute r.passed?
+    refute_predicate r, :passed?
 
     r.report
 
@@ -111,7 +164,7 @@ class TestMinitestReporter < Minitest::Test
   end
 
   def test_passed_eh_skipped_verbose
-    r.first.options[:verbose] = true
+    r.options[:verbose] = true
 
     r.start
     r.results << skip_test
@@ -138,7 +191,31 @@ class TestMinitestReporter < Minitest::Test
     assert_equal 0, r.assertions
   end
 
+  def test_record_pass_with_metadata
+    reporter = self.r
+
+    def reporter.metadata
+      @metadata
+    end
+
+    def reporter.record result
+      super
+      @metadata = result.metadata if result.metadata?
+    end
+
+    r.record passing_test_with_metadata
+
+    exp = { :meta => :data }
+    assert_equal exp, reporter.metadata
+
+    assert_equal ".", io.string
+    assert_empty r.results
+    assert_equal 1, r.count
+    assert_equal 0, r.assertions
+  end
+
   def test_record_fail
+    fail_test = self.fail_test
     r.record fail_test
 
     assert_equal "F", io.string
@@ -148,6 +225,7 @@ class TestMinitestReporter < Minitest::Test
   end
 
   def test_record_error
+    error_test = self.error_test
     r.record error_test
 
     assert_equal "E", io.string
@@ -157,6 +235,7 @@ class TestMinitestReporter < Minitest::Test
   end
 
   def test_record_skip
+    skip_test = self.skip_test
     r.record skip_test
 
     assert_equal "S", io.string
@@ -165,25 +244,6 @@ class TestMinitestReporter < Minitest::Test
     assert_equal 0, r.assertions
   end
 
-  def normalize_output output
-    output.sub!(/Finished in .*/, "Finished in 0.00")
-    output.sub!(/Loaded suite .*/, 'Loaded suite blah')
-
-    output.gsub!(/ = \d+.\d\d s = /, ' = 0.00 s = ')
-    output.gsub!(/0x[A-Fa-f0-9]+/, '0xXXX')
-    output.gsub!(/ +$/, '')
-
-    if windows? then
-      output.gsub!(/\[(?:[A-Za-z]:)?[^\]:]+:\d+\]/, '[FILE:LINE]')
-      output.gsub!(/^(\s+)(?:[A-Za-z]:)?[^:]+:\d+:in/, '\1FILE:LINE:in')
-    else
-      output.gsub!(/\[[^\]:]+:\d+\]/, '[FILE:LINE]')
-      output.gsub!(/^(\s+)[^:]+:\d+:in/, '\1FILE:LINE:in')
-    end
-
-    output
-  end
-
   def test_report_empty
     r.start
     r.report
@@ -200,7 +260,6 @@ class TestMinitestReporter < Minitest::Test
       0 runs, 0 assertions, 0 failures, 0 errors, 0 skips
     EOM
 
-
     assert_equal exp, normalize_output(io.string)
   end
 
@@ -221,7 +280,6 @@ class TestMinitestReporter < Minitest::Test
       1 runs, 0 assertions, 0 failures, 0 errors, 0 skips
     EOM
 
-
     assert_equal exp, normalize_output(io.string)
   end
 
@@ -246,7 +304,6 @@ class TestMinitestReporter < Minitest::Test
       1 runs, 0 assertions, 1 failures, 0 errors, 0 skips
     EOM
 
-
     assert_equal exp, normalize_output(io.string)
   end
 
@@ -276,15 +333,41 @@ class TestMinitestReporter < Minitest::Test
     assert_equal exp, normalize_output(io.string)
   end
 
-  def restore_env
-    old_value = ENV["MT_NO_SKIP_MSG"]
-    ENV.delete "MT_NO_SKIP_MSG"
-    
-    yield
-  ensure
-    ENV["MT_NO_SKIP_MSG"] = old_value
-  end
+  def test_report_error__sse
+    r.start
+    r.record system_stack_error_test
+    r.report
 
+    exp = clean <<-EOM
+      Run options:
+
+      # Running:
+
+      E
+
+      Finished in 0.00
+
+        1) Error:
+      Minitest::Test#woot:
+      SystemStackError: 274 -> 12
+          a
+          b
+          c
+           +->> 67 cycles of 4 lines:
+           | aa
+           | ab
+           | ac
+           | ad
+           +-<<
+          d
+          e
+          f
+
+      1 runs, 0 assertions, 0 failures, 1 errors, 0 skips
+    EOM
+
+    assert_equal exp, normalize_output(io.string)
+  end
 
   def test_report_skipped
     r.start
@@ -310,4 +393,48 @@ class TestMinitestReporter < Minitest::Test
 
     assert_equal exp, normalize_output(io.string)
   end
+
+  def test_report_failure_uses_backtrace_filter
+    filter = Minitest::BacktraceFilter.new
+    def filter.filter _bt
+      ["foo.rb:123:in `foo'"]
+    end
+
+    with_backtrace_filter filter do
+      r.start
+      r.record fail_test
+      r.report
+    end
+
+    exp = "Minitest::Test#woot [foo.rb:123]"
+
+    assert_includes io.string, exp
+  end
+
+  def test_report_failure_uses_backtrace_filter_complex_sorbet
+    backtrace = <<~EOBT
+      /Users/user/.gem/ruby/3.2.2/gems/minitest-5.20.0/lib/minitest/assertions.rb:183:in `assert'
+      example_test.rb:9:in `assert_false'
+      /Users/user/.gem/ruby/3.2.2/gems/sorbet-runtime-0.5.11068/lib/types/private/methods/call_validation.rb:256:in `bind_call'
+      /Users/user/.gem/ruby/3.2.2/gems/sorbet-runtime-0.5.11068/lib/types/private/methods/call_validation.rb:256:in `validate_call'
+      /Users/user/.gem/ruby/3.2.2/gems/sorbet-runtime-0.5.11068/lib/types/private/methods/_methods.rb:275:in `block in _on_method_added'
+      example_test.rb:25:in `test_something'
+      /Users/user/.gem/ruby/3.2.2/gems/minitest-5.20.0/lib/minitest/test.rb:94:in `block (3 levels) in run'
+      /Users/user/.gem/ruby/3.2.2/gems/minitest-5.20.0/lib/minitest/test.rb:191:in `capture_exceptions'
+      /Users/user/.gem/ruby/3.2.2/gems/minitest-5.20.0/lib/minitest/test.rb:89:in `block (2 levels) in run'
+      ... so many lines ...
+    EOBT
+
+    filter = Minitest::BacktraceFilter.new %r%lib/minitest|gems/sorbet%
+
+    with_backtrace_filter filter do
+      begin
+        assert_equal 1, 2
+      rescue Minitest::Assertion => e
+        e.set_backtrace backtrace.lines.map(&:chomp)
+
+        assert_match "example_test.rb:25", e.location
+      end
+    end
+  end
 end
